#include "WebServer.h"
#include "Animationen.h"
#include <TimeLib.h>
#include "NTPClient.h"
#include "wortuhr_func.h"
#include "LedStrip.h"
#include <ESP8266WebServer.h>
#include "Global.h"

ESP8266WebServer WebServer(80);

String onmessage = "&nbsp;";
String offmessage = "&nbsp;";


void StartWebServer() {
	Serial.println("Start WebServer");
	WebServer.on("/", HandleRoot);
	WebServer.on("/UpdateNTP", HandleUpdateNTP);
	WebServer.on("/getTime", HandleGetTime);
	WebServer.on("/dimmer", HandleToggleDimmer);
	WebServer.on("/configAnim", HandleConfigAnim);
	WebServer.on("/configBrightness", HandleConfigBrightness);
	WebServer.on("/configColor", HandleConfigColor);
	WebServer.on("/Logs", HandleLogs);

	WebServer.on("/brightness", HandleBrightness);
	WebServer.on("/color", HandleColor);
	WebServer.on("/animColor", HandleAnimColor);
	WebServer.on("/saveAnim", HandleSaveAnim);
	WebServer.on("/getLogs", HandleGetLogs);

	WebServer.on("/test", HandleTest);
	WebServer.on("/refresh", HandleRefresh);
	WebServer.on("/clear", HandleClear);
	WebServer.on("/anim", HandleAnimStart);
	WebServer.on("/matrix", HandleMatrixTest);
  WebServer.on("/restart", HandleRestart);
	WebServer.begin();

}



void ShowPage(String content, String script, String style) {
	String page = FPSTR(HTTP_HEADER);
	page += script;
	page += FPSTR(HTTP_SCRIPT_END);
	page += style;
	page += FPSTR(HTTP_CONTAINER);

	// replacements
	content.replace("{ct}", String(hour()) + ":" + String(minute()));
	content.replace("{ct12}", String(hourFormat12()) + ":" + String(minute()));
	content.replace("{menu}", FPSTR(HTTP_MENU));
	content.replace("{onmessage}", onmessage);
	content.replace("{offmessage}", offmessage);
	content.replace("{brightness}", String(brightness));
	content.replace("{color}", "#FF0000");
	onmessage = "&nbsp;";
	offmessage = "&nbsp;";
	page += content;
	page += FPSTR(HTTP_END);
	WebServer.send(200, "text/html", page);
}

String MainMenu() {

}

void HandleRoot() {
	Serial.println("HandleRoot");
	String content = FPSTR(HTTP_MAIN);
	content += "{menu}";

	ShowPage(content, FPSTR(HTTP_MAIN_SCRIPT), FPSTR(HTTP_STYLE));
}

void HandleUpdateNTP() {
	Serial.println("HandleUpdateNTP");
	updateTime();
	wordClock();
	onmessage = "Uhrzeit wurde aktualisiert!";

	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleToggleDimmer() {
	Serial.println("HandleToggleDimmer");
	if (toggleDimmer()) {
		onmessage = "Dimmer eingeschaltet";
	}
	else {
		offmessage = "Dimmer ausgeschaltet";
	}
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleConfigAnim() {
	Serial.println("HandleConfigAnim");
	String content = FPSTR(HTTP_MENU_BACK);
	content += FPSTR(HTTP_ANIM_CONFIG);

	for (int i = 0; i < sizeof(ANIM_VALUES); i++) {
		if (i == GLOBAL::anim) {
			content.replace("{" + String(i) + "_selected}", "selected");
		} else {
			content.replace("{" + String(i) + "_selected}", "");
		}
	}

	ShowPage(content, FPSTR(HTTP_ANIM_SCRIPT), FPSTR(HTTP_STYLE));
}

void HandleSaveAnim() {
	Serial.println("HandleSaveAnim");

	String colorstr = WebServer.arg("animColor").substring(1);
	int result;
	result = strtoul(colorstr.c_str(), NULL, 16);
	GLOBAL::animColor = RgbColor(HtmlColor(result));
	GLOBAL::animColorBefore = RgbColor(HtmlColor(result));
	GLOBAL::animColorBefore.Darken(180);
	Log("Anim Farbe: " + colorstr);

	GLOBAL::anim = WebServer.arg("anim").toInt();
	Log("Animation gespeichert: " + String(GLOBAL::anim));

	onmessage = "Einstellungen gespeichert";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleConfigBrightness() {
	Serial.println("HandleConfigBrightness");

	String content = FPSTR(HTTP_BRIGHTNESS_CONFIG);
	content += FPSTR(HTTP_MENU_BACK);

	ShowPage(content, FPSTR(HTTP_BRIGHTNESS_SCRIPT), FPSTR(HTTP_STYLE));
}

void HandleConfigColor() {
	Serial.println("HandleConfigColor");

	String content = FPSTR(HTTP_COLOR_CONFIG);
	content += FPSTR(HTTP_MENU_BACK);

	ShowPage(content, FPSTR(HTTP_COLOR_SCRIPT), FPSTR(HTTP_STYLE));
}


void HandleColor() {
	Serial.println("HandleColor");
	String colorstr = WebServer.arg("value").substring(1);
	int result;
	result = strtoul(colorstr.c_str(), NULL, 16);
	setColor(result);
	Log("Farbe: " + colorstr);
	onmessage = "Einstellungen gespeichert";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleAnimColor() {
	Serial.println("HandleAnimColor");
	String colorstr = WebServer.arg("value").substring(1);
	int result;
	result = strtoul(colorstr.c_str(), NULL, 16);
	GLOBAL::animColor = RgbColor(HtmlColor(result));
	GLOBAL::animColorBefore = RgbColor(HtmlColor(result));
	GLOBAL::animColorBefore.Darken(80);
	Log("Anim Farbe: " + colorstr);
	onmessage = "Einstellungen gespeichert";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleBrightness() {
	Serial.println("HandleBrightness");

	setBrightness(WebServer.arg("value").toInt());

	onmessage = "Einstellungen gespeichert";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleLogs() {
	Serial.println("HandleGetLogs");
	String content = "<textarea readonly id=\"logs\" cols=\"45\" wrap=\"off\"></textarea>";
	content += FPSTR(HTTP_MENU_BACK);

	ShowPage(content, FPSTR(HTTP_LOGS_SCRIPT), FPSTR(HTTP_STYLE));
}

void HandleTest() {
	Serial.println("HandleTest");

	onmessage = "Test wird ausgef�hrt!";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
	LedTest();
	test();
}

void HandleRefresh() {
	Log("HandleRefresh");

	onmessage = "Refresh wird ausgef�hrt!";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
	wordClock(true);
}

void HandleClear() {
	clearLeds();
	strip.Show();
	onmessage = "Anzeige zur�ckgesetzt";
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleAnimStart() {
	playAnimation();
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}
void HandleMatrixTest() {
	MatrixTest();
	WebServer.sendHeader("Location", String("/"), true);
	WebServer.send(302, "text/plain", "");
}

void HandleGetLogs() {
	WebServer.send(200, "text/html", GetLogs());
}

void HandleGetTime() {
	char timestr[17];
	int int_hour = hourFormat12();
	if (summertime_EU(year(), month(), day(), hour(), 1)) {
		if (int_hour == 12) {
			int_hour = 1;
		}
		else {
			int_hour = int_hour + 1;
		}
	}
	sprintf(timestr, "%02d.%02d. - %02d:%02d:%02d", day(), month(), int_hour, minute(), second());

	WebServer.send(200, "text/html", timestr);
}

void HandleRestart() {
  ESP.restart();
}

WebServer::WebServer()
{
	
}


WebServer::~WebServer()
{
}
