#include <NeoPixelAnimator.h>
#include <NeoPixelBrightnessBus.h>
#include <Timer.h>
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <TimeLib.h>
#include <ESP8266mDNS.h>
#include <ArduinoOTA.h>

#include "Global.h"
#include "NTPClient.h"
#include "Logging.h"
#include "WebServer.h"
#include "LedStrip.h"
#include "Animationen.h"

Timer t;


const char* WIFI_SSID = "MKT";
const char* WIFI_PW = "11MarKat06";
const int LED_PIN = LED_BUILTIN;

int GLOBAL::anim = 0;
RgbwColor GLOBAL::animColor = RgbwColor(60, 60, 255);
RgbwColor GLOBAL::animColorBefore = RgbwColor(60, 60, 255);


/*************************** Sketch Code ************************************/

void setup() {
	// -------  Serial & PIN Setting
	Serial.begin(74880);
	//Serial.begin(115200);
	pinMode(LED_PIN, OUTPUT);

	// -------- WLAN Connection
	Log("Connecting to WLAN");
	WiFi.mode(WIFI_STA);
	WiFi.hostname("wortuhr");
	WiFi.begin(WIFI_SSID, WIFI_PW);

	while (WiFi.status() != WL_CONNECTED) {
		digitalWrite(LED_PIN, HIGH);
		delay(200);
		digitalWrite(LED_PIN, LOW);
		delay(200);
		Serial.print(".");
	}
	Serial.println();

	ArduinoOTA.onStart([]() {

	});
	ArduinoOTA.onEnd([]() {
		Serial.println("\nEnd");
	});
	ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
		Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
	});
	ArduinoOTA.onError([](ota_error_t error) {
		Serial.printf("Error[%u]: ", error);
		if (error == OTA_AUTH_ERROR) {
			Serial.println("Auth Failed");
		}
		else if (error == OTA_BEGIN_ERROR) {
			Serial.println("Begin Failed");
		}
		else if (error == OTA_CONNECT_ERROR) {
			Serial.println("Connect Failed");
		}
		else if (error == OTA_RECEIVE_ERROR) {
			Serial.println("Receive Failed");
		}
		else if (error == OTA_END_ERROR) {
			Serial.println("End Failed");
		}
	});
	ArduinoOTA.setHostname("wortuhr");
	ArduinoOTA.begin();

	IPAddress HTTPS_ServerIP = WiFi.localIP(); // Obtain the IP of the Server
	Log("Server IP is: " + HTTPS_ServerIP.toString()); // Print the IP to the monitor window
	long rssi = WiFi.RSSI();
	Log("WLAN Signal: " + String(rssi));

	// WebServer Initialisierung
	StartWebServer();

	// Zeitabruf per hour(), minute(), second(), now()
	setSyncInterval(99999999);
	setSyncProvider(getNtpTime);
	t.every(86400000, updateTime);


	//t.every(1000, timeOut);
	t.every(10000, wordClock);
	//delay(10000);

	// this resets all the neopixels to an off state
	strip.Begin();
	strip.Show();
}

void loop() {
	WebServer.handleClient();
	ArduinoOTA.handle();
	t.update();
	delay(100);
}



void timeOut() {
	Serial.println(String(hour()) + ":" + String(minute()));
}
